/*
 * Created on Apr 1, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.ibm.mda.uxjsf.transform;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;

import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.ibm.xde.mda.delegates.MdaAssociationEnd;
import com.ibm.xde.mda.delegates.MdaAttribute;
import com.ibm.xde.mda.delegates.MdaClass;
import com.ibm.xde.mda.delegates.MdaDependency;
import com.ibm.xde.mda.delegates.MdaModel;
import com.ibm.xde.mda.delegates.MdaOperation;
import com.ibm.xde.mda.delegates.MdaPackage;
import com.ibm.xde.mda.util.MdaOption;
import com.ibm.xde.mda.util.Signature;
import com.ibm.xde.mda.util.TypeExpression;
import com.rational.rxe.IRXEClass;
import com.rational.rxe.IRXEClassProxy;
import com.rational.rxe.IRXEDependencies;
import com.rational.rxe.IRXEDependency;
import com.rational.rxe.IRXEElement;

/**
 * @author jim conallen
 *
 */
public class JSFConfiguration {

	private Document document = null;
	private Element root = null;
	private Element applicationElement = null;
	private Element factoryElement = null;
	private Element componentElement = null;
	private Element converterElement = null;
	private Element managedBeanElement = null;
	private Element navigationRuleElement = null;
	private Element referencedBeanElement = null;
	private Element renderKitElement = null;
	private Element lifecycleElement = null;
	private Element validatorElement = null;
	
	private MdaPackage sitePackage = null;
	private MdaModel codeModel = null;
	private IProject project = null;
	
	/*
	 * managed bean methods
	 */

	public void addManagedBean(MdaClass bean) throws IOException {
		// if a code model is specified then create/update the implementation classes
		if( codeModel != null ) {
			// then we should either create the implementation class, 
			// or if one is specified verify that all the getters/setters are there.
			MdaClass implementation = findManagedBeanImplementation(bean);
			if( implementation == null ) {
				String typeExpStr = null;
				boolean packageSpecified = !bean.isTaggedValueADefault("JSF_1a", "package name");
				boolean classNameSpecified = !bean.isTaggedValueADefault("JSF_1a", "class name");  
				if( packageSpecified || classNameSpecified ) {
					String packageName = bean.getTaggedValueAsString("JSF_1a", "package name");
					String className = bean.getTaggedValueAsString("JSF_1a", "class name");

					if( packageSpecified ) {
						typeExpStr = packageName;
					} else {
						typeExpStr = ""; 
					}
					if( classNameSpecified ) {
						typeExpStr += "." + UX_to_JSF_DesignTransformer.normalizeName(className);
					} else {
						typeExpStr += "." + UX_to_JSF_DesignTransformer.normalizeName(bean.getName());
					}
				}
				if( typeExpStr != null ) {
					TypeExpression typeExp = new TypeExpression(typeExpStr);
					implementation = codeModel.findClass(typeExp, MdaOption.CREATE_IF_MISSING);
					// now create a dependency from the content class to the implementation
					MdaDependency dep = bean.findDependencyTo(implementation, MdaOption.CREATE_IF_MISSING);
					dep.addStereotype("JSF_1a","implementation");
					updateImplementation(bean, implementation);
				}
			} else {
				updateImplementation(bean, implementation);
			}
		}

		String id = bean.getID();
		Element managedBeanElement = findElementByNameId(root, "managed-bean", id );
		if( managedBeanElement == null ){
			managedBeanElement = newManagedBean(bean);	
		} else {
			updateManagedBean(bean, managedBeanElement);
		}
		
	}
	
	private void updateImplementation(MdaClass bean, MdaClass impl) throws IOException {
		// for each of the attributes in the bean, make sure there is a getter and setter
		// don't worry about an attribute per say, unless both the get and set are both missing
		MdaAttribute attributes[] = bean.getMdaAttributes();
		for(int i=0;i<attributes.length;i++) {
			MdaAttribute beanAttr = attributes[i];
			String name = beanAttr.getName();
			String typeExp = beanAttr.getTypeExpression();

			String setterStr = "set" + name.substring(0,1).toUpperCase() + name.substring(1) + "(" + typeExp + "):void";
			Signature sigSetter = new Signature(setterStr); 
			MdaOperation setter = impl.findOperation(sigSetter, MdaOption.NONE); 

			String getterStr = "get" + name.substring(0,1).toUpperCase() + name.substring(1) + "():" + typeExp;
			Signature sigGetter = new Signature(getterStr); 
			MdaOperation getter = impl.findOperation(sigGetter, MdaOption.NONE); 
			
			if( setter == null && getter == null ) {
				// need to create the attr and ops
				MdaAttribute implAttr = impl.findAttribute(name, MdaOption.CREATE_IF_MISSING);
				implAttr.setTypeExpression(typeExp);
				getter = implAttr.createGetter();
				setter = implAttr.createSetter();
			} else if( setter == null ) {
				setter = impl.createOperation(sigSetter, MdaOption.NONE);
			} else if( getter == null ) {
				getter = impl.createOperation(sigGetter, MdaOption.NONE);
			}
		}
		
	}
	
	private Element newManagedBean(MdaClass bean) throws IOException {		
		Element mbElm = document.createElement("managed-bean");       // Create element
		mbElm.setAttribute("id", bean.getID() );
		
		// description
		String descr = bean.getDescription(); 
		if( descr != null && descr.length()>0 ) {
			Element description = document.createElement("description");
			Text text = document.createTextNode(descr);
			description.appendChild(text);
			mbElm.appendChild(description);
		}
		
		// managed-bean-name
		Element nameElement = document.createElement("managed-bean-name");
		String beanName = bean.getName();
		nameElement.appendChild(document.createTextNode(beanName));
		mbElm.appendChild(nameElement);
		
		// managed-bean-class
		MdaClass implementation = findManagedBeanImplementation(bean);
		String implType = ""; 
		if( implementation != null ) {
			TypeExpression exp = new TypeExpression(implementation);
			implType = exp.toString();
		} 
		Element classElement = document.createElement("managed-bean-class");
		classElement.appendChild(document.createTextNode(implType));
		mbElm.appendChild(classElement);
		
		// managed-bean-scope
		String scope = bean.getTaggedValueAsString("JSF_1a", "scope" );
		Element scopeElement = document.createElement("managed-bean-scope");
		scopeElement.appendChild(document.createTextNode(scope));
		mbElm.appendChild(scopeElement);
		
		// managed-property | map-entries | list-entries  however this version only supports managed-property
		MdaAttribute[] attributes = bean.getMdaAttributes();
		for(int i=0;i<attributes.length;i++) {
			MdaAttribute attr = attributes[i];
			appendBeanProperty( attr, mbElm ) ; 
		}

		// attach the completed new node to the root right before any existing ref bean elem
		if( navigationRuleElement != null ) {
			root.insertBefore( mbElm, navigationRuleElement );
		} else if( referencedBeanElement != null ) {
			root.insertBefore( mbElm, referencedBeanElement );
		} else if( renderKitElement != null ) {
			root.insertBefore( mbElm, renderKitElement);
		} else if( lifecycleElement != null ) {
			root.insertBefore( mbElm, lifecycleElement);
		} else if( validatorElement != null ) {
			root.insertBefore( mbElm, validatorElement);
		} else {
			root.appendChild(mbElm);
		}
		// set internal navRule list
		if( managedBeanElement == null ) {
			managedBeanElement = mbElm;
		}
		
		return mbElm;
	}
	
	
	private void appendBeanProperty( MdaAttribute attr, Element mbElm)  throws IOException {
		//(description*, display-name*, icon*, property-name, property-class?, (map-entries|null-value|value|list-entries))
		Element propElm = document.createElement("managed-bean-property");       // Create element
		propElm.setAttribute("id", attr.getID() );
		
		// description
		String descr = attr.getDescription(); 
		if( descr != null && descr.length()>0 ) {
			Element description = document.createElement("description");
			Text text = document.createTextNode(descr);
			description.appendChild(text);
			propElm.appendChild(description);
		}
		
		// property-name
		Element nameElement = document.createElement("property-name");
		String propName = attr.getName();
		nameElement.appendChild(document.createTextNode(propName));
		propElm.appendChild(nameElement);
				
		// property-class
		String className = attr.getTypeExpression();
		if( className != null && className.length()>0 ) {
			Element classElement = document.createElement("property-class");
			classElement.appendChild(document.createTextNode(className));
			propElm.appendChild(classElement);
		}
		
		// value, null-value
		String value = attr.getDefaultValue();
		if( value != null && value.length()>0 ) {
			Element valueElement = document.createElement("value");
			valueElement.appendChild(document.createTextNode(value));
			propElm.appendChild(valueElement);
		} else {
			Element nullValueElement = document.createElement("null-value");
			propElm.appendChild(nullValueElement);
		}
				
		mbElm.appendChild(propElm);
	}
	
	private MdaClass findManagedBeanImplementation(MdaClass bean) throws IOException {
		MdaClass impl = null;
		IRXEDependencies dependencies = bean.getDependencyRelationships();
		int count = dependencies.getCount();
		for(int i=1;i<=count;i++) {
			IRXEDependency rxeDep = dependencies.getDependencyByPosition(i); 
			MdaDependency mdaDep = new MdaDependency(rxeDep);
			if( mdaDep.isStereotyped("JSF_1a", "implementation" ) ) {
				impl = mdaDep.getSupplierMdaClass();
				break;
			}
		}
		return impl; 
	}
	
	private void updateManagedBean(MdaClass bean, Element managedBeanElement ) throws IOException {
		Element descriptionElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "description");
		//Element displayNameElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "display-name");
		//Element iconElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "icon");
		Element managedBeanNameElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "managed-bean-name");
		Element managedBeanClassElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "managed-bean-class");
		Element managedBeanScopeElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "managed-bean-scope");
		Element managedPropertyElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "managed-bean-property");
		Element mapEntriesElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "map-entries");
		Element listEntriesElement = (Element) getFirstNodeOfTypeAndName(managedBeanElement, Node.ELEMENT_NODE, "list-entries");
		
		String description = bean.getDescription();
		String managedBeanName = bean.getName();
		MdaClass implementation = findManagedBeanImplementation(bean);
		String managedBeanClass = ""; 
		if( implementation != null ) {
			TypeExpression exp = new TypeExpression(implementation);
			managedBeanClass = exp.toString();
		} 
		String scope = bean.getTaggedValueAsString("JSF_1a", "scope" );

		// update description
		if( descriptionElement == null ) {
			if( description != null && description.length()>0 ) {
				descriptionElement = document.createElement("description");
				descriptionElement.appendChild(document.createTextNode(description));
				NodeList nodes = managedBeanElement.getChildNodes();
				if( nodes.getLength()>0) {
					// make this the first child
					managedBeanElement.insertBefore(descriptionElement, nodes.item(0));
				} else {
					// we should never get hear - unless the config file was edited manually.
					managedBeanElement.appendChild(descriptionElement);
				}
			}
		}
		
		// managed-bean-name
		if( managedBeanNameElement == null ) {
			// this should not happen, however if it is does we need to create the element
			managedBeanNameElement = document.createElement("managed-bean-name");
			managedBeanNameElement.appendChild(document.createTextNode(managedBeanName));
			if( managedBeanClassElement != null ) {
				managedBeanElement.insertBefore(managedBeanNameElement, managedBeanClassElement);
			} else if( managedBeanScopeElement != null ){
				managedBeanElement.insertBefore(managedBeanNameElement, managedBeanScopeElement);
			} else if( managedPropertyElement != null ){
				managedBeanElement.insertBefore(managedBeanNameElement, managedPropertyElement);
			} else if( mapEntriesElement != null ){
				managedBeanElement.insertBefore(managedBeanNameElement, mapEntriesElement);
			} else if( listEntriesElement != null ){
				managedBeanElement.insertBefore(managedBeanNameElement, listEntriesElement);
			} else {
				managedBeanElement.appendChild(managedBeanNameElement);
			}
		} else {
			String currentBeanName = getTextContent(managedBeanNameElement);
			if( currentBeanName == null || !currentBeanName.equals(managedBeanName)) {
				setTextContent(managedBeanName, managedBeanNameElement);
			}
		}
		
		// managed-bean-class
		if( managedBeanClassElement == null ) {
			// this should not happen, however if it is does we need to create the element
			managedBeanClassElement = document.createElement("managed-bean-class");
			managedBeanClassElement.appendChild(document.createTextNode(managedBeanClass));
			if( managedBeanScopeElement != null ){
				managedBeanElement.insertBefore(managedBeanClassElement, managedBeanScopeElement);
			} else if( managedPropertyElement != null ){
				managedBeanElement.insertBefore(managedBeanClassElement, managedPropertyElement);
			} else if( mapEntriesElement != null ){
				managedBeanElement.insertBefore(managedBeanClassElement, mapEntriesElement);
			} else if( listEntriesElement != null ){
				managedBeanElement.insertBefore(managedBeanClassElement, listEntriesElement);
			} else {
				managedBeanElement.appendChild(managedBeanClassElement);
			}
		} else {
			String currentBeanClass = getTextContent(managedBeanClassElement);
			if( currentBeanClass == null || !currentBeanClass.equals(managedBeanClass)) {
				setTextContent(managedBeanClass, managedBeanClassElement);
			}
		}
		
		// managed-bean-scope
		if( managedBeanScopeElement == null ) {
			// this should not happen, however if it is does we need to create the element
			managedBeanScopeElement = document.createElement("managed-bean-scope");
			managedBeanScopeElement.appendChild(document.createTextNode(scope));
			if( managedPropertyElement != null ){
				managedBeanElement.insertBefore(managedBeanScopeElement, managedPropertyElement);
			} else if( mapEntriesElement != null ){
				managedBeanElement.insertBefore(managedBeanScopeElement, mapEntriesElement);
			} else if( listEntriesElement != null ){
				managedBeanElement.insertBefore(managedBeanScopeElement, listEntriesElement);
			} else {
				managedBeanElement.appendChild(managedBeanScopeElement);
			}
		} else {
			String currentScope = getTextContent(managedBeanScopeElement);
			if( currentScope != null && !currentScope.equals(scope)) {
				setTextContent(scope, managedBeanScopeElement);
			}
		}
		
		// properties
		MdaAttribute attributes[] = bean.getMdaAttributes();
		for(int i=0;i<attributes.length;i++) {
			MdaAttribute attr = attributes[i];
			String id = attr.getID();
			Element elm = findElementByNameId(managedBeanElement, "managed-bean-property", id);
			if( elm != null ) {
				updateManagedProperty(attr, elm);
			} else {
				appendBeanProperty(attr, managedBeanElement);
			}
		}
	}
	
	private void updateManagedProperty(MdaAttribute attr, Element managedPropertyElement) throws IOException {
		//(description*, display-name*, icon*, property-name, property-class?, (map-entries|null-value|value|list-entries))
		Element descriptionElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "description");
		//Element displayNameElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "display-name");
		//Element iconElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "icon");
		Element propertyNameElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "property-name");
		Element propertyClassElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "property-class");
		Element mapEntriesElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "map-entries");
		Element nullValueElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "null-value");
		Element valueElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "value");
		Element listEntriesElement = (Element) getFirstNodeOfTypeAndName(managedPropertyElement, Node.ELEMENT_NODE, "list-entries");
		
		String description = attr.getDescription();
		String propertyName = attr.getName();
		String propertyClass = attr.getTypeExpression();
		String value = attr.getDefaultValue();
		
		// update description
		if( descriptionElement == null ) {
			if( description != null && description.length()>0 ) {
				descriptionElement = document.createElement("description");
				descriptionElement.appendChild(document.createTextNode(description));
				NodeList nodes = managedPropertyElement.getChildNodes();
				if( nodes.getLength()>0) {
					// make this the first child
					managedPropertyElement.insertBefore(descriptionElement, nodes.item(0));
				} else {
					// we should never get hear - unless the config file was edited manually.
					managedPropertyElement.appendChild(descriptionElement);
				}
			}
		}
		
		// property-name
		if( propertyNameElement == null ) {
			// this should not happen, however if it is does we need to create the element
			propertyNameElement = document.createElement("property-name");
			propertyNameElement.appendChild(document.createTextNode(propertyName));
			if( propertyClassElement != null ) {
				managedPropertyElement.insertBefore(propertyNameElement, propertyClassElement);
			} else if( mapEntriesElement != null ){
				managedPropertyElement.insertBefore(propertyNameElement, mapEntriesElement);
			} else if( nullValueElement != null ){
				managedPropertyElement.insertBefore(propertyNameElement, nullValueElement);
			} else if( valueElement != null ){
				managedPropertyElement.insertBefore(propertyNameElement, valueElement);
			} else if( listEntriesElement != null ){
				managedPropertyElement.insertBefore(propertyNameElement, listEntriesElement);
			} else {
				managedPropertyElement.appendChild(propertyNameElement);
			}
		} else {
			String currentPropertyName = getTextContent(propertyNameElement);
			if( currentPropertyName != null && !currentPropertyName.equals(propertyName)) {
				setTextContent(propertyName, propertyNameElement);
			}
		}
		
		// property-class
		if( propertyClassElement == null ) {
			// this should not happen, however if it is does we need to create the element
			propertyClassElement = document.createElement("property-class");
			propertyClassElement.appendChild(document.createTextNode(propertyClass));
			if( mapEntriesElement != null ){
				managedPropertyElement.insertBefore(propertyClassElement, mapEntriesElement);
			} else if( nullValueElement != null ){
				managedPropertyElement.insertBefore(propertyClassElement, nullValueElement);
			} else if( valueElement != null ){
				managedPropertyElement.insertBefore(propertyClassElement, valueElement);
			} else if( listEntriesElement != null ){
				managedPropertyElement.insertBefore(propertyClassElement, listEntriesElement);
			} else {
				managedPropertyElement.appendChild(propertyClassElement);
			}
		} else {
			String currentPropertyClass = getTextContent(propertyClassElement);
			if( currentPropertyClass != null && !currentPropertyClass.equals(propertyName)) {
				setTextContent(propertyClass, propertyClassElement);
			}
		}
		
		// value
		if( mapEntriesElement != null ) managedPropertyElement.removeChild(mapEntriesElement);
		if( listEntriesElement != null ) managedPropertyElement.removeChild(listEntriesElement);
		if( value == null || value.length()==0 ) {
			if( valueElement != null ) managedPropertyElement.removeChild(valueElement);
			if( nullValueElement == null ) {
				nullValueElement = document.createElement("null-value");
				managedPropertyElement.appendChild(nullValueElement);
			}
		} else {
			if( nullValueElement != null ) managedPropertyElement.removeChild(nullValueElement);
			if( valueElement == null ) {
				valueElement = document.createElement("value");
				valueElement.appendChild(document.createTextNode(value));
				managedPropertyElement.appendChild(valueElement);
			} else {
				setTextContent(value, valueElement);
			}
		}
	}
	
	
	
	/*
	 * Navigation Rule methods
	 */
	
	public void addNavigationRule(Screen from) throws IOException {
		String id = from.getId();
		Element navElm = findElementByNameId(root,"navigation-rule", id );
		if( navElm == null ) {
			// create it
			newNavRule(from);
		} else {
			// check and modify if necessary (update)
			updateNavRule(from,navElm);
		}
	}
	
	private void newNavRule(Screen from) throws IOException{
		Element navRule = document.createElement("navigation-rule");       // Create element
		navRule.setAttribute("id", from.getId() );
		
		String descr = from.getDescription(); 
		if( descr != null && descr.length()>0 ) {
			Element description = document.createElement("description");
			Text text = document.createTextNode(descr);
			description.appendChild(text);
			navRule.appendChild(description);
		}
		Element fromViewId = document.createElement("from-view-id");
		String path = from.getRelativeScreenPath();
		Text text = document.createTextNode(path);
		fromViewId.appendChild(text);
		navRule.appendChild(fromViewId);

		// for all associations to other screens, and that have a rolename specified
		// create a nav case
		
		MdaAssociationEnd[] navCases = from.getNavCaseEnds();
		for(int i=0;i<navCases.length;i++) {
			addNavCase(navCases[i],navRule);
		}
		
		// attach the completed new node to the root right before any existing ref bean elem
		if( referencedBeanElement != null ) {
			root.insertBefore( navRule, referencedBeanElement );
		} else if( renderKitElement != null ) {
			root.insertBefore( navRule, renderKitElement);
		} else if( lifecycleElement != null ) {
			root.insertBefore( navRule, lifecycleElement);
		} else if( validatorElement != null ) {
			root.insertBefore( navRule, validatorElement);
		} else {
			root.appendChild(navRule);
		}
		// set internal navRule list
		if( navigationRuleElement == null ) {
			navigationRuleElement = navRule;
		}
	}
	
	private void addNavCase(MdaAssociationEnd end, Element navRule) throws DOMException, IOException {
		Element navCase = document.createElement("navigation-case");
		navCase.setAttribute("id", end.getID() );
		String descr = end.getDescription(); 
		if( descr != null && descr.length()>0 ) {
			Element description = document.createElement("description");
			Text text = document.createTextNode(descr);
			description.appendChild(text);
			navCase.appendChild(description);
		}
		Element fromOutcome = document.createElement("from-outcome");
		String name = end.getRole();
		Text text = document.createTextNode(name);
		fromOutcome.appendChild(text);
		navCase.appendChild(fromOutcome);
		
		IRXEElement rxeElm = end.getElement();
		if( rxeElm.getObjectKind().equalsIgnoreCase("class") ) {
			IRXEClass rxeCls = new IRXEClassProxy(rxeElm);
			MdaClass mdaCls = new MdaClass(rxeCls);
			Screen targetScreen = new Screen(mdaCls);
			String targetPath = targetScreen.getRelativeScreenPath();

			Element toViewId = document.createElement("to-view-id");
			text = document.createTextNode(targetPath);
			toViewId.appendChild(text);
			navCase.appendChild(toViewId);
		}
		navRule.appendChild(navCase);
	}
	
	private void updateNavRule( Screen from, Element navRule) throws IOException {
		// assume that id is already set, otherwise we'd be creating a new rule
		String descr = from.getDescription();
		Element descriptionElement = null; 
		// in this mapping we only allow one description element, even though
		// the DTD specifies that multiple are allowed. 
		Node descriptionNode = getFirstNodeOfTypeAndName(navRule, Node.ELEMENT_NODE, "description");
		if( descriptionNode != null ) {
			// then one exists already
			descriptionElement = (Element) descriptionNode;
			if( descr == null || descr.length()==0 ) {
				// remove it
				navRule.removeChild(descriptionElement);
			} else {
				// we are going to update it if 
				setTextContent(descr, descriptionElement);
			}
		} 
		// skip over the displayname, and icon for this mapping
		Element fromViewId = (Element) getFirstNodeOfTypeAndName(navRule, Node.ELEMENT_NODE, "from-view-id");
		if( fromViewId == null ) {
			// this indicates a potentially corrupt config file, although valid against the DTD
			// We need to create it
			String path = from.getRelativeScreenPath();
			fromViewId = document.createElement("from-view-id");
			fromViewId.appendChild( document.createTextNode(path));
			// now find a place to insert the node
			Node firstNavCase = getFirstNodeOfTypeAndName(navRule, Node.ELEMENT_NODE, "navigation-case");
			if( firstNavCase == null ) {
				 navRule.appendChild(fromViewId);
			} else {
				navRule.insertBefore(fromViewId, firstNavCase);
			}
		}
		
		// for all associations to other screens, and that have a rolename specified
		// create/update a nav case
		MdaAssociationEnd[] navCases = from.getNavCaseEnds();
		for(int i=0;i<navCases.length;i++) {
			MdaAssociationEnd navCaseEnd = navCases[i];
			String id = navCaseEnd.getID();
			Element navCaseNode = findNavCaseElement(navRule, id);
			if( navCaseNode == null ) {
				// then this is a new one need to add it.
				addNavCase(navCaseEnd,navRule);
			} else {
				updateNavCase(navCaseEnd, navCaseNode);
			}
			
		}
	}
	
	private void updateNavCase(MdaAssociationEnd navCaseEnd, Element navCaseElement) throws IOException {
		Element descriptionElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "description");
		Element displayNameElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "display-name");
		Element iconElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "icon");
		Element fromActionElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "from-action");
		Element fromOutcomeElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "from-outcome");
		Element toViewIdElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "to-view-id"); 
		Element redirectElement = (Element) getFirstNodeOfTypeAndName(navCaseElement, Node.ELEMENT_NODE, "redirect"); 
		
		String description = navCaseEnd.getDescription();
		String fromOutcome = navCaseEnd.getRole();
		Screen toScreen = new Screen( navCaseEnd.getElement() );
		String toViewId = toScreen.getRelativeScreenPath();
		
		// description
		if( description != null && description.length()==0 ) description = null; 
		if( descriptionElement == null ) {
			if( description != null ) {
				// create the element, always the first in the child
				descriptionElement = document.createElement("description");
				descriptionElement.appendChild(document.createTextNode(description));
				NodeList nodes = navCaseElement.getChildNodes();
				if( nodes.getLength()==0 ) {
					navCaseElement.insertBefore(descriptionElement, nodes.item(0));
				} else {
					navCaseElement.appendChild(descriptionElement);
				}
			}
		} else {
			if( descriptionElement.hasAttribute("id") && description == null) {
				navCaseElement.removeChild(descriptionElement);
			}
		}

		// from-outcome
		if( fromOutcomeElement == null ) {
			// this should not happen, however it is does we need to create the element
			fromOutcomeElement = document.createElement("from-outcome");
			fromOutcomeElement.appendChild(document.createTextNode(fromOutcome));
			if( toViewIdElement!= null ) {
				navCaseElement.insertBefore(fromOutcomeElement, toViewIdElement);
			} else if( redirectElement != null ) {
				navCaseElement.insertBefore(fromOutcomeElement, redirectElement);
			} else {
				navCaseElement.appendChild(fromOutcomeElement);
			}
		} else {
			String currentFromOutcome = getTextContent(fromOutcomeElement);
			if( !currentFromOutcome.equals(fromOutcome) ) {
				setTextContent(fromOutcome, fromOutcomeElement);
			}
		}
		
		// to-view-id
		if( toViewIdElement == null ) {
			// this should not happen, however it is does we need to create the element
			toViewIdElement = document.createElement("to-view-id");
			toViewIdElement.appendChild(document.createTextNode(toViewId));
			if( redirectElement != null ) {
				navCaseElement.insertBefore(toViewIdElement, redirectElement);
			} else {
				navCaseElement.appendChild(toViewIdElement);
			}
		} else {
			String currentToViewId = getTextContent(toViewIdElement);
			if( !currentToViewId.equals(toViewId) ) {
				setTextContent(toViewId, toViewIdElement);
			}
		}
	}
	
	private Element findNavCaseElement(Element navRuleElement, String id) {
		Element elm = findElementByNameId(navRuleElement, "navigation-case", id );
		return elm;
	}
	
	/*
	 * DOM utility methods
	 */
	private Element findElementByNameId(Element parent, String name, String id) {
		Element elm = null;
		NodeList elements = parent.getElementsByTagName(name);
		int count = elements.getLength();
		for(int i=0;i<count;i++) {
			Node n = elements.item(i);
			if( n.getNodeType() == Node.ELEMENT_NODE ) {
				Element e = (Element) n;
				String eId = e.getAttribute("id");
				if( eId != null && eId.equals(id) ) {
					elm = e;
					break;
				}
			}
		}
		return elm;
	}
	
	private String getTextContent(Element element) {
		String text = null;
		Node textNode = getFirstNodeOfType(element, Node.TEXT_NODE);
		if( textNode != null ) {
			text = textNode.getNodeValue();
		}
		return text;
	}
	
	private String setTextContent(String text, Element element) {
		Node textNode = getFirstNodeOfType(element, Node.TEXT_NODE);
		if( textNode == null ){
			textNode = document.createTextNode(text);
			element.appendChild(textNode);
		} else {
			textNode.setNodeValue(text);
		}
		return text;
	}
	
	private Element getFirstChildElementByName(Element parent, String elementName) {
		Element element = (Element) getFirstNodeOfTypeAndName(parent, Node.ELEMENT_NODE, elementName);
		return element;
	}
	
	private Node getFirstNodeOfType(Element parent, short nodeType ) {
		Node node = null;
		NodeList children = parent.getChildNodes();
		int count = children.getLength();
		for(int i=0;i<count;i++) {
			Node n = children.item(i);
			if( n.getNodeType() == nodeType ) {
				node = n; 
				break;
			}
		}
		return node;
	}
	
	private Node getFirstNodeOfTypeAndName(Element parent, short nodeType, String nodeName) {
		Node node = null;
		NodeList children = parent.getChildNodes();
		int count = children.getLength();
		for(int i=0;i<count;i++) {
			Node n = children.item(i);
			if( n.getNodeType() == nodeType && n.getNodeName().equals(nodeName)) {
				node = n; 
				break;
			}
		}
		return node;
	}
	
	
	public void saveConfiguration(IFile file) throws IOException, CoreException {
		OutputFormat format  = new OutputFormat( document );   //Serialize DOM
		format.setIndenting(true);
		StringWriter writer = new StringWriter();
		XMLSerializer serial = new XMLSerializer( writer, format );
		serial.asDOMSerializer();                            // As a DOM Serializer
		serial.serialize( document.getDocumentElement() );
		ByteArrayInputStream inStream = new ByteArrayInputStream(writer.toString().getBytes());
		if( file.exists() )
			file.setContents(inStream, true, false, null);
		else 
			file.create(inStream, true, null);
	}
	
	private void openConfigFile(IFile file) throws IOException, CoreException {
		DOMParser parser = new DOMParser();
		try {
			InputStream inStream = file.getContents();
			InputSource inSource = new InputSource(inStream); 
			parser.parse(inSource);
			document = parser.getDocument();
			root = document.getDocumentElement();
			
			NodeList childNodes = root.getChildNodes();
			int count = childNodes.getLength();
			for(int i=0;i<count;i++) {
				Node childNode = childNodes.item(i);
				String nodeName = childNode.getNodeName();
				int nodeType = childNode.getNodeType();
				if( nodeType == Node.ELEMENT_NODE ) {
					if( applicationElement == null && nodeName.equals("application") ) {
						applicationElement = (Element) childNode;
					}
					if( factoryElement == null && nodeName.equals("factory") ){
						factoryElement = (Element) childNode;
					}
					if(  componentElement == null && nodeName.equals("component") ){
						componentElement = (Element) childNode;
					}
					if( converterElement == null && nodeName.equals("converter") ) {
						converterElement = (Element) childNode;
					}
					if( managedBeanElement == null && nodeName.equals("managed-bean") ) {
						managedBeanElement = (Element) childNode;
					}
					if( navigationRuleElement == null && nodeName.equals("navigation-rule") ) {
						navigationRuleElement = (Element) childNode;
					}
					if( referencedBeanElement == null && nodeName.equals("referenced-bean") ) {
						referencedBeanElement = (Element) childNode;
					}
					if( renderKitElement == null && nodeName.equals("render-kit") ) {
						renderKitElement = (Element) childNode;
					}
					if( lifecycleElement == null && nodeName.equals("lifecycle") ) {
						lifecycleElement = (Element) childNode;
					}
					if( validatorElement == null && nodeName.equals("validator") ) {
						validatorElement = (Element) childNode;
					}
				}
				
				String nodeValue = childNode.getNodeValue();
			}
			
		} catch (SAXException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 */
	private void initializeConfigFile(IFile file) throws Exception {
		// copy the intitial file from the templates folder of this plugin
		File initialConfigFile = UX_to_JSF_DesignTransformer.getFileInPlugin("templates/faces-config.xml");
		UX_to_JSF_DesignTransformer.copyFile(initialConfigFile, file);
	}


	/*
	 * Constructors
	 */

	public JSFConfiguration(IFile file, MdaPackage sitePackage, MdaModel codeModel, IProject project) throws Exception {
		this.sitePackage = sitePackage;
		this.codeModel = codeModel;
		this.project = project;
		//IFile file = project.getFile(filename);
		if( !file.exists() ) {
			initializeConfigFile(file);
		} 
		openConfigFile(file);
	}
	

	private JSFConfiguration(){}
}
