/*
 * Created on Apr 6, 2004
 *
 */
package com.ibm.mda.uxjsf.transform;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;

/**
 * @author jim conallen
 *
 */
public class JSP {
	
	private IFile file = null;
	private HashMap tags = new HashMap(); 
	private Vector fragments = new Vector();  
	private int titleFragmentIndex = -1;
	private int startViewIndex = -1;
	private int startFormIndex = -1;
	private int endViewIndex = -1;
	private int endFormIndex = -1;
	
	public JSP() {
	}
	
	public Fragment getFragment(int index) {
		Fragment fragment = (Fragment) fragments.get(index);
		return fragment;	
	}
	
	public Fragment insertNewTag(String tagString) {
		Fragment tag = new Fragment(tagString);
		String id = tag.getTagId();
		if( id != null && id.length()>0 && endFormIndex>0 ) {
			fragments.insertElementAt(tag, endFormIndex);
			tags.put(id, tag);
		}
		return tag;
	}
	
	
	public void removeFragment(Fragment fragment) {
		if( fragment.isTag() ) {
			// then there is the potential for tested tags/fragments.
			if( fragment.isEmptyTag() ) {
				fragments.remove(fragment);
			} else if( fragment.isStartTag() ){
				// only remove start tags - don't allow the removal of an end tag
				String tagName = fragment.getTagName();
				int depth = 1;
				int index = fragments.indexOf(fragment)+1;
				boolean removing = true;
				while( removing && index<fragments.size() ) {
					Fragment nestedFragment = (Fragment) fragments.get(index);
					if( nestedFragment.isTag() ) {
						String nestedName = nestedFragment.getTagName();
						if( tagName.equals(nestedName) ) {
							if( nestedFragment.isStartTag() ) depth++;
							if( nestedFragment.isEndTag() ) {
								depth--;
								if( depth<=0 ) removing = false; 
							}
						}
					}
					fragments.remove(nestedFragment);
				}
			}
		} else {
			fragments.remove(fragment);
		}
	}
	
	public Fragment getTagbyId(String id){
		Fragment frag = (Fragment) tags.get(id);
		return frag;
	}
	
	public void saveFile(IFile file) throws CoreException {
		String fileString = "";
		Iterator i = fragments.iterator();
		while(i.hasNext()) {
			Fragment f = (Fragment)i.next();
			fileString += f.getText();
		}
		ByteArrayInputStream bytes = new ByteArrayInputStream(fileString.getBytes()); 
		file.setContents(bytes ,false, true, null);
	}

	public void parseFile(IFile file) throws IOException, CoreException{
		this.file = file;
		tags = new HashMap(); 
		fragments = new Vector();  

		int prev = -1;
		char c;
		int col = 1;
		int line = 1;
		int i = 0;  
		boolean inString = false;
		boolean inScriptlet = false;
		boolean inTag = false;
		Fragment fragment = newFragment("");
		
		InputStream in = file.getContents();
		int b = in.read();
		c = (char)b;
		while( b>=0 ) {
			if( c=='\n' && c=='\r' && c=='\f' && inString ) {
				fragment.append(c);
				inString = false;
			} else if( Character.isLetter(c) && prev=='<' && !inString && !inScriptlet ){
				// in a tag, start new fragment with this char
				fragment = newFragment("<" + c);
				inTag = true;	
			} else if( c=='/' && prev=='<' && !inString && !inTag ) {
				// end tag, start new empty fragment.
				fragment = newFragment("</");
				inTag = true;
			} else if( c=='>' && !inString && inTag ) {
				// end tag, start new empty fragment.
				fragment.append(">");
				addTag(fragment);
				inTag = false;
				fragment = newFragment("");
			} else if( c=='>' && prev=='%' && inScriptlet && !inString ) {
				fragment.append(c);
				inScriptlet = false;
			} else if( c=='%' && prev=='<' && !inString ) {
				fragment.append("<%");
				inScriptlet = true;
			} else if( c=='!' && prev=='<' && !inString ) {
				fragment.append("<!");
			} else if( c=='\"' && prev !='\\' && !inString ) {
				fragment.append(c);
				inString = false;
			} else if( c=='\"' && prev!='\\' && !inString ) {
				fragment.append(c);
				inString = true;
			} else if( c!='<' ){
				fragment.append(c);
			}
			
			prev = c;
			b = in.read();
			c = (char) b;
		}
	}
	
	private void addTag(Fragment fragment) {
		if( fragment.isTag() ) {
			int index = fragments.indexOf(fragment);
			String name = fragment.getTagName();
			
			if( startViewIndex<0  ){
				if( name.equals("f:view") ) {
					if( fragment.isStartTag() ) {
						startViewIndex = index;
					}
				}
			} else if( endViewIndex<0 ) {
				if( name.equals("f:view") ) {
					if( fragment.isEndTag() ) {
						endViewIndex = index;
					}
				}
			}

			if( startFormIndex<0  ){
				if( name.equals("h:form") ) {
					if( fragment.isStartTag() ) {
						startFormIndex = index;
					}
				}
			} else if( endFormIndex<0 ) {
				if( name.equals("h:form") ) {
					if( fragment.isEndTag() ) {
						endFormIndex = index;
					}
				}
			}
			
			String id = fragment.getTagId();
			if( id != null && id.length()>0) {
				tags.put(id, fragment);
			}
		}
	}
	
	public Fragment[] getFragments(){
		Fragment fragArray[] = {};
		return (Fragment[]) fragments.toArray(fragArray);  
	}
	
	public Map getTags(){
		return tags;  
	}
	
	
	/*
	 * Utility methods
	 */
	
	private Fragment newFragment(String text) {
		Fragment fragment = new Fragment(text);
		fragments.add(fragment);
		return fragment;
	}
	
	private Fragment newFragment(char c) {
		Fragment fragment = new Fragment(c);
		fragments.add(fragment);
		return fragment;
	}
	

	private static String findId(String tag) {
		StringTokenizer st = new StringTokenizer(tag, " \t\n\r\f<>\\\"=", true);
		String prevToken = null;
		String token = null;
		String idValue = null;
		boolean inString = false;
		boolean lookingForIdValue = false;
		while( st.hasMoreTokens() ) {
			prevToken = token;
			token = st.nextToken();
			
			if( token.equals("\"") && !prevToken.equals("\\") ) {
				inString  = !inString;
				if( !inString ) {
					if( lookingForIdValue == true) {
						lookingForIdValue = false;
						break;
					} 
				}
			}
			
			if( !inString && token.equals("id") ) lookingForIdValue = true; 
			
			if( lookingForIdValue ) {
				if( inString ) {
					if( idValue == null ) 
						idValue = ""; 
					else 
						idValue += token;
				} 
			}
		}
		
		return idValue;		
	}

}
