package com.ibm.mda.uxjsf.transform;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;

import com.ibm.mda.uxjsf.UxJsfPlugin;
import com.ibm.mda.uxjsf.ui.UX_to_JSF_DesignParameters;
import com.ibm.xde.mda.Transformer;
import com.ibm.xde.mda.delegates.MdaClass;
import com.ibm.xde.mda.delegates.MdaModel;
import com.ibm.xde.mda.delegates.MdaPackage;
import com.ibm.xde.mda.util.Log;
import com.ibm.xde.mda.util.MdaOption;
import com.ibm.xde.mda.util.TypeExpression;


/**
 * @author jim conallen
 *
 */
public class UX_to_JSF_DesignTransformer extends Transformer {
	private final static String CLIENT_DESCRIPTION = "UX to JSF Design";
	private static Log _myLog = null;
	private MdaPackage sitePackage = null;
	private MdaModel codeModel = null;
	private JSFConfiguration config;
	private IProject project = null;
	private IFolder webFolder = null;
	
	/**
	 * 
	 */
	public UX_to_JSF_DesignTransformer() {
		super();
		
		_myLog = createLog("UX to JSF Design");
		setPreferenceStore(UxJsfPlugin.getDefault().getPreferenceStore());
	}
	
	private static Log getLog() {
		return _myLog;
	}

	/** 
	 * @see com.ibm.xde.mda.Transformer#transform()
	 * @throws Exception
	 */
	public void transform() throws Exception {
		writeln("\n\nBeginning UX to JSF Trasnformation.");
		String inputModelFilename = getStringParameter(UX_to_JSF_DesignParameters.P_INPUT_MODEL_FILENAME);
		MdaModel uxModel = openModel("ux", inputModelFilename);
		if( project == null ) setProject(uxModel);

		String configFilename = getStringParameter(UX_to_JSF_DesignParameters.P_CONFIG_FILENAME);
		IFile file = project.getFile(configFilename);

		String web = getStringParameter(UX_to_JSF_DesignParameters.P_WEB_FOLDER);
		webFolder = project.getFolder(web);
		
		if( codeModel == null ) {
			String codeModelFilename = getStringParameter(UX_to_JSF_DesignParameters.P_CODE_MODEL_FILENAME);
			if( codeModelFilename != null && codeModelFilename.length()>0 ) {
				codeModel = openModel("code", codeModelFilename);
				if( codeModel != null ) {
					if( !codeModel.canRTE() ) {
						codeModel.close();
						codeModel = null;
					}
				} 
			}
		}
		
		String site = getStringParameter(UX_to_JSF_DesignParameters.P_SITE_PACKAGE);
		TypeExpression pkgExp = new TypeExpression(site);
		if( sitePackage == null ) sitePackage = uxModel.findPackage(pkgExp, MdaOption.NONE );
		try{
			config = new JSFConfiguration(file, sitePackage, codeModel, project);

			// lets walk through the model elements and create the JSPs and config file elements
			transformModelElements(sitePackage);
		
	
			// now save the configuration file 
			config.saveConfiguration(file);
		
		} catch( Exception e ) {
			writeln("Unable to create or access configuration file.");
		}
		
		writeln("Transformation UX to JSF Design is completed.");
	}


	private void transformModelElements(MdaPackage pkg) throws Exception {
		MdaClass[] modelElements = pkg.getMdaClasses(MdaOption.NONE);
		for(int i=0;i<modelElements.length;i++){
			MdaClass mdaCls = modelElements [i];
			String name = mdaCls.getName();
			if( mdaCls.isStereotyped("UX_2a", "screen") ) {
				writeln("Processing screen: " + name);
				Screen screen = new Screen(mdaCls);
				screen.writeJSP(webFolder);
				// now add any nav rules to the config file
				writeln("Adding navigation rules to configuration for screen: " + name);
				config.addNavigationRule(screen);
			} else if( mdaCls.isStereotyped("UX_2a", "content") ) {
				writeln("Adding managed bean: " + name);
				config.addManagedBean(mdaCls);
			} 
		}
	}
	
	private void setProject(MdaModel model) throws IOException {
		String filename  = model.getPath();
		IWorkspaceRoot root = UxJsfPlugin.getWorkspace().getRoot();
		IPath path = new Path(filename);
		IContainer container = root.getContainerForLocation(path);
		project = container.getProject();
	}

	public static void copyFile(File in, File out) throws Exception {
		FileInputStream fis  = new FileInputStream(in);
		FileOutputStream fos = new FileOutputStream(out);
		byte[] buf = new byte[1024];
		int i = 0;
		while((i=fis.read(buf))!=-1) {
			fos.write(buf, 0, i);
		}
		fis.close();
		fos.close();
	}
    
	private File getFilesFolder() {
		File files = getFileInPlugin("files");
		return files;
	}

	public static void copyFile(File src, IFile target) throws Exception {
		if( src.isFile() && !target.exists() ) {
			String newFileString = target.getLocation().toOSString();
			File targetFile = new File(newFileString);
			FileInputStream fis = new FileInputStream(src);
			target.create(fis,true,null);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.xde.mda.Transformer#validate()
	 */
	public boolean validate() throws IOException {
		// we should check to make sure all assumptions about the parameters are
		// met before we start a trasnformation.
		writeln("\n\nValidating transformation parameters.");

		boolean valid = true;
		String inputModelFilename = getStringParameter(UX_to_JSF_DesignParameters.P_INPUT_MODEL_FILENAME);
		MdaModel uxModel = openModel("ux", inputModelFilename);
		// make sure that the UX_2a profile and JSF_1a profiles have been applied.
		String appliedProfiles = uxModel.getAppliedProfilesAsString(",");
		if( appliedProfiles.indexOf("UX_2a")<0 ) {
			writeln("\tThe UX profile (UX_2a) has not been applied to the specified UX model.");
			valid = false;
		}
		if( appliedProfiles.indexOf("JSF_1a")<0 ) {
			writeln("\tThe JSF profile (JSF_1a) has not been applied to the specified UX model.");
			valid = false;
		}
		
		if( project == null ) setProject(uxModel);
		if( project == null ) {
			writeln("\tUnable to determine owning project for the UX model.  The model must be owned by a valid Eclipse project.");
			valid = false;
		}
		
		String site = getStringParameter(UX_to_JSF_DesignParameters.P_SITE_PACKAGE);
		if( site == null && site.length()==0 ) {
			writeln("\tThe site parameter is required.");
			valid = false;
		} else {
			TypeExpression pkgExp = new TypeExpression(site);
			if( sitePackage == null ) sitePackage = uxModel.findPackage(pkgExp, MdaOption.NONE );
			if( sitePackage == null ) {
				writeln("\tUnable to find the site: " + site + " in the UX model.");
				valid = false;
			}
		}
		 
		String web = getStringParameter(UX_to_JSF_DesignParameters.P_WEB_FOLDER);
		if( web == null && web.length()==0 ) {
			writeln("\tThe web site location parameter is required.");
			valid = false;
		} else {
			if( webFolder == null ) webFolder = project.getFolder(web);
			if( webFolder == null ) {
				writeln("\tUnable to resolve the web directory with the value: " + web);
				valid = false;
			}
		}
		
		String codeModelFilename = getStringParameter(UX_to_JSF_DesignParameters.P_CODE_MODEL_FILENAME);
		if( codeModelFilename != null && codeModelFilename.length()>0 ) {
			MdaModel codeModel = openModel("code", codeModelFilename);
			if( codeModel != null ) {
				IPath codeLocation = new Path(codeModelFilename);
				IContainer container = project.getWorkspace().getRoot().getContainerForLocation(codeLocation);
				IProject codeProject = container.getProject();
				if( !project.equals(codeProject) ){
					writeln("\tUX model and code model must belong to the same project");
					valid = false;
				}
				if( !codeModel.canRTE() ) {
					writeln("\tThe model specified as a code model is not a valid XDE code model: " + codeModelFilename);
					valid = false;
				}
			} else {
				writeln("\tUnable to open the specified code model: " + codeModelFilename);
				valid = false;
			}
		}
		 
		if( valid ) 
			writeln("Transformation parameters validated.");
		else 
			writeln("Transformation parameters invalid transformation aborted.");
			
		return true;
	}

	/* (non-Javadoc)
	 * @see com.ibm.xde.mda.Transformer#verify()
	 */
	public boolean verify() {
		writeln("\n\nVerification has not been completed.");
		return true;
	}
	
	public static void writeln(String message) {
		getLog().writeln(message);
	}
	
	public void write(String message) {
		getLog().write(message);
	}
	
	/**
	 * Data-driven approach which uses current preference settings. Will be 
	 * called from
	 * - menu item ("Transform"); or
	 * - ANT extension classes 
	 */
	public static void launchTransformation(int actions) {
		UX_to_JSF_DesignTransformer transformer = new UX_to_JSF_DesignTransformer();
		performAction(
			transformer,
			CLIENT_DESCRIPTION,
			isModelWritten(),
			getLog(),
			actions);
	}
	
	/**
	 * Interactive approach which allows client to invoke transform with 
	 * parameters different from current preference settings. Will be called 
	 * from menu item ("Custom Transform...") which invokes a dialog where 
	 * parameters can be alternated.
	 * 
	 * Sole purpose of this method is to simplify transform invocation from 
	 * "Custom Transform..." dialog. Don't call this method for any other 
	 * purposes unless you fully understand how to create a preference store and 
	 * set it up with correct values. See launchTransformation(void).
	 * 
	 * @param store
	 * @param actions (values: ACTION_VALIDATE, ACTION_TRANSFORM, ACTION_VERIFY)
	 * @return true if success during execution or false otherwise
	 */
	public static boolean launchTransformation(IPreferenceStore store, int actions) {
		UX_to_JSF_DesignTransformer transformer = new UX_to_JSF_DesignTransformer();
		transformer.setPreferenceStore(store);
		return performAction(
			transformer,
			CLIENT_DESCRIPTION,
			isModelWritten(),
			getLog(),
			actions);
	}
	
	/**
	 * @return true if a model is modified during transformation
	 */
	public static boolean isModelWritten() {
		return true;
	}
	
	/**
	 * Finds the absolute path to a file given its relative Eclipse path
	 * 
	 * @param path - relative Eclipse's path to a resource
	 * @return absolute path to the resource on the file system
	 */
	public static File getFileInPlugin(String path) {
		try {
			URL installURL = UxJsfPlugin.getDefault().getDescriptor().find(new Path(path));
			URL localURL = Platform.asLocalURL(installURL);
			return new File(localURL.getFile());
		} catch (IOException e) {
			return null;
		}
	}
	/**
	 * open MDA supplied helper model by its short name.
	 * 
	 * @param modelName - short model name
	 * @return
	 * @throws Exception
	 */
	private MdaModel openLocalModel(String modelName) throws Exception {
		File file = getFileInPlugin("models/" + modelName + ".mdx");
		String modelPath = file.getAbsolutePath();
	 
		// open the model privately
		return openModelPrivately(modelName, modelPath);
	}

	
	/** 
	 * When a model is opened, no actions should be started because basic XDE GUI 
	 * services may need to start write actions in responce to opening a model.
	 * To prevent action protocol violations we temporarily close the action, 
	 * open a model and then reopen the action.
	 * 
	 * Alternatively you can use openModelPrivately to avoid the hassle.
	 * 
	 * @see com.ibm.xde.mda.Transformer#openModel(java.lang.String, java.lang.String)
	 */
	public MdaModel openModel(String name, String filename) throws IOException {
		MdaModel result = null; 
		
		// complete action before opening a model and close it after that.
		getRXEApplication().completeAction();
		result = super.openModel(name, filename);
		if (isModelWritten()) {
			getRXEApplication().startWriteAction(CLIENT_DESCRIPTION);
		}
		else {
			getRXEApplication().startReadAction(CLIENT_DESCRIPTION);
		}
		
		return result;
	}

	public static String normalizeName(String aName) {
		String name = "";
		int count = aName.length();
		for(int i=0;i<count;i++) {
			char c = aName.charAt(i);
			if( Character.isLetter(c) || (Character.isLetterOrDigit(c) && i>0) ) {
				name += c;
			} else {
				name += "_";
			}
		}
		return name;
	}

	public static String encode(String aQuotedValue) {
		String value = "";
		int count = aQuotedValue.length();
		for(int i=0;i<count;i++) {
			char c = aQuotedValue.charAt(i);
			if( c == '"') {
				value += "&quote";
			} else if( c == '&') {
				value += "&amp";
			} else {
				value += c;
			} 
		}
		return value;
	}
}
