/*
 * Created on Nov 17, 2003
 */
package com.ibm.mda.uxjsf.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

import com.ibm.xde.mda.Transformer;
import com.ibm.mda.uxjsf.transform.UX_to_JSF_DesignTransformer;


/**
 * @author dubauski
 *
 * XDE 1.5, 
 * IBM Rational Software 2003
 */
public class UX_to_JSF_DesignDialog extends Dialog implements SelectionListener {
	UX_to_JSF_DesignDialogPage _page = new UX_to_JSF_DesignDialogPage("UX to JSF Design dialog");
	Button _buttonValidate = null;
	Button _buttonTransform = null;
	Button _buttonVerify = null;	 

	/**
	 * @param parentShell
	 */
	public UX_to_JSF_DesignDialog(Shell parentShell) {
		super(parentShell);
		setShellStyle(SWT.SHELL_TRIM);
	}
	
	/**
	 * @see org.eclipse.jface.window.Window#configureShell(Shell)
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("UX to JSF Design dialog");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		// create parameters group
		Group group1 = new Group(parent, SWT.NULL);
		group1.setText("Parameters: ");
		Composite fields = _page.createMainPanel(group1);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		group1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group1.setLayout(layout);
		
		fields.setLayout(new GridLayout());
		fields.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		((GridLayout) fields.getLayout()).numColumns = 3;
		
		
		// Create additional controls
		Group group2 = new Group(parent, SWT.NULL);
		group2.setText("Actions to be executed: ");
		layout = new GridLayout();
		layout.numColumns = 1;
		group2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group2.setLayout(layout);
		
		_buttonValidate = new Button(group2, SWT.CHECK);
		_buttonValidate.setText("Validate parameters");
		_buttonValidate.addSelectionListener(this);
		
		_buttonTransform = new Button(group2, SWT.CHECK);
		_buttonTransform.setText("Execute transformation");
		_buttonTransform.addSelectionListener(this);
		
		_buttonVerify = new Button(group2, SWT.CHECK);
		_buttonVerify.setText("Verify transformation results");
		_buttonVerify.addSelectionListener(this);
		
		_buttonValidate.setSelection(true);
		_buttonTransform.setSelection(true);
		_buttonVerify.setSelection(true);

		// adjust main composite
		((GridLayout) parent.getLayout()).marginWidth = 5;

		return parent;
	}
	
	/**
	 *
	 * Initializes the location and size of this window's SWT shell after it has
	 * been created. This framework method is called by the create framework
	 * method. The default implementation calls getInitialSize and
	 * getInitialLocation and passes the results to Shell.setBounds.
	 * This is only done if the bounds of the shell have not already been
	 * modified. Subclasses may extend or reimplement.
	 */
	protected void initializeBounds() {
	  Shell shell = getShell();

	  Rectangle screenBounds = shell.getDisplay().getBounds();
	  Point dialogSize = getInitialSize();

	  // Put the dialog in the center
	  int widthScreen = screenBounds.width;
	  int heightScreen = screenBounds.height;
	  
	  dialogSize.x = widthScreen / 3;

	  int dialogX = widthScreen / 2 - dialogSize.x / 2;
	  int dialogY = heightScreen / 2 - dialogSize.y / 2;
	  shell.setLocation(dialogX, dialogY);
	  shell.setSize(dialogSize);
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		// do not change this code. It is needed to pass correct parameters 
		// to Transformer.
		_page.performOk();
		
		// obtain the state of action requests (validate, transform, verify)
		// and pass it to the launchTransformation() 
		int actions = 0;
		
		if (_buttonValidate.getSelection()) {
			actions |= Transformer.ACTION_VALIDATE;
		}
		if (_buttonTransform.getSelection()) {
			actions |= Transformer.ACTION_TRANSFORM;
		}
		if (_buttonVerify.getSelection()) {
			actions |= Transformer.ACTION_VERIFY;
		}
		
		if (UX_to_JSF_DesignTransformer.launchTransformation(
				_page.getPreferenceStore(),	actions)) {
			MessageDialog.openInformation(
				getShell(),
				"UX to JSF Design",
				"UX to JSF Design has been executed successfully");
		}
		else {
			MessageDialog.openInformation(
				getShell(),
				"UX to JSF Design",
				"UX to JSF Design has been executed, but problems" + 
				" were encountered. Check Output view for details");
		}
			
		super.okPressed();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		Button b = getButton(IDialogConstants.OK_ID);
		b.setEnabled(true);
		if (!_buttonValidate.getSelection() &&
			!_buttonTransform.getSelection() &&
			!_buttonVerify.getSelection()) {
			b.setEnabled(false);
		}
	}

	
}
